import { createSlice, PayloadAction } from "@reduxjs/toolkit";
import type { RootState } from "./store";
import { getGraphStatistics, GraphStatistics } from "../../statistics";

export interface GraphQueryResultFromBackendPayload {
  queryID: string;
  result: {
    payload: GraphQueryResultFromBackend;
    type: string;
  };
}

export type NodeAttributes = { [key: string]: unknown };

export interface GraphQueryResultFromBackend {
  nodes: {
    _id: string;
    label: string;
    attributes: NodeAttributes;
  }[];

  edges: {
    _id: string;
    label: string;
    attributes: NodeAttributes;
    from: string;
    to: string;
  }[];

  // TODO: Also include type in node and edge
  // TODO: The backend should send all the different entitytypes and relationtypes in the result
}

export type Node = {
  _id: string;
  label: string;
  attributes: NodeAttributes;
  mldata?: any; // FIXME
  /* type: string[]; */
};
export type Edge = {
  attributes: NodeAttributes;
  from: string;
  to: string;
  _id: string;
  label: string;
  /* type: string; */
};
export type UniqueEdge = {
  attributes: NodeAttributes;
  source: string;
  target: string;
  count: number;
};

// Define a type for the slice state
export type GraphQueryResult = {
  metaData?: GraphStatistics;
  nodes: Node[];
  edges: Edge[];
  queryingBackend: boolean;
};

// Define the initial state using that type
export const initialState: GraphQueryResult = {
  metaData: undefined,
  nodes: [],
  edges: [],
  queryingBackend: false,
};

export const graphQueryBackend2graphQuery = (payload: GraphQueryResultFromBackend) => {
  // Assign ids to a hidden attribute _id to better identify them
  payload.nodes.forEach((node, index) => {
    node._id = node._id || (node as any).id;
  });
  payload.edges.forEach((edge, index) => {
    edge._id = edge._id || (edge as any).id;
  });

  // Only keep one node and one edge per id. This is also done in the backend, but we do it here as well to be sure.
  const nodeIDs = new Set(payload.nodes.map((node) => node._id));
  const edgeIDs = new Set(payload.edges.map((edge) => edge._id));
  const nodes = [...nodeIDs].map((nodeID) => payload.nodes.find((node) => node._id === nodeID) as unknown as Node);
  const edges = [...edgeIDs].map((edgeID) => payload.edges.find((edge) => edge._id === edgeID) as unknown as Edge);

  const metaData = getGraphStatistics(payload);

  return { metaData, nodes, edges };
};

export const graphQueryResultSlice = createSlice({
  name: "graphQueryResult",
  initialState,
  reducers: {
    setNewGraphQueryResult: (state, action: PayloadAction<GraphQueryResultFromBackend | any>) => {
      let payload: GraphQueryResultFromBackend;

      if (action.payload.result && action.payload.result.payload) {
        payload = action.payload.result.payload;
      } else if (action.payload.payload && action.payload.type) {
        payload = action.payload.payload;
      } else if (action.payload.nodes && action.payload.edges) {
        payload = action.payload;
      } else {
        console.error("Invalid payload structure in setNewGraphQueryResult:", action.payload);
        return;
      }

      const { metaData, nodes, edges } = graphQueryBackend2graphQuery(payload);

      // Assign new state
      state.metaData = metaData;
      state.nodes = nodes;
      state.edges = edges;
      state.queryingBackend = false;
    },
    resetGraphQueryResults: (state) => {
      // Assign new state
      state.metaData = undefined;
      state.nodes = [];
      state.edges = [];
      state.queryingBackend = false;
    },
    queryingBackend: (state) => {
      state.queryingBackend = true;
    },
  },
});

export const { setNewGraphQueryResult, resetGraphQueryResults, queryingBackend } = graphQueryResultSlice.actions;

// Other code such as selectors can use the imported `RootState` type
export const selectGraphQueryResult = (state: RootState) => state.graphQueryResult;
export const selectGraphQueryResultNodes = (state: RootState) => state.graphQueryResult.nodes;
export const selectGraphQueryResultLinks = (state: RootState) => state.graphQueryResult.edges;
export const selectGraphQueryResultMetaData = (state: RootState) => state.graphQueryResult.metaData;

export default graphQueryResultSlice.reducer;
