import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import { RootState } from '../store';

export type InsightType = 'report' | 'alert';

export const InsightConditionMap: Record<string, InsightCondition["operator"]> = {
  'Greater than': '>',
  'Equal than': '==',
  'Smaller than': '<',
  'Not equal': '!=',
  'Greater or equal': '>=',
  'Smaller or equal': '<=',
};
export const InsightConditionMapReverse = Object.fromEntries(
  Object.entries(InsightConditionMap).map(([key, value]) => [value, key])
);

export type InsightCondition = {
  nodeLabel: string;
  property: string;
  statistic: string;
  operator: ">" | "==" | "<" | "!=" | ">=" | "<=";
  value: number;
};

export type InsightRequest = {
  name: string;
  description: string;
  recipients: string[];
  frequency: string;
  template: string;
  saveStateId: string;
  userId?: number;
  status?: boolean;
  conditionsCheck: InsightCondition[]
  type: 'report' | 'alert';
  alarmMode: 'disabled' | 'diff' | 'conditional' | 'always';
}

export type InsightResponse = {
  id: number;
  previousResultHash?: string;
  lastProcessedAt?: string;
  createdAt: string;
  updatedAt: string;
} & InsightRequest;

type InsightState = {
  insights: InsightResponse[];
};

const initialState: InsightState = {
  insights: [],
};

const insightSharingSlice = createSlice({
  name: 'insightSharing',
  initialState,
  reducers: {
    setInsights(state, action: PayloadAction<InsightResponse[]>) {
      state.insights = action.payload;
    },
    addInsight: (state, action: PayloadAction<InsightResponse>) => {
      if (!action.payload) {
        console.error('Received null payload in addInsight');
        return;
      }

      const type = action.payload.type;
      if (!type) {
        console.error('Insight type is missing in payload', action.payload);
        return;
      }

      state.insights.push(action.payload);
    },
    updateInsight(state, action: PayloadAction<InsightResponse>) {
      const index = state.insights.findIndex((i) => i.id === action.payload.id);
      if (index !== -1) {
        state.insights[index] = { ...action.payload };
      } else {
        state.insights.push(action.payload);
      }
    },
    deleteInsight(state, action: PayloadAction<{ id: number }>) {
      const index = state.insights.findIndex((i) => i.id === action.payload.id);
      if (index !== -1) {
        state.insights.splice(index, 1);
      }
    },
  },
});

export const { setInsights, addInsight, updateInsight, deleteInsight } = insightSharingSlice.actions;

export const selectReports = (state: RootState): InsightResponse[] =>
  state.insightSharing.insights?.filter((i) => i.type === 'report') ?? [];
export const selectAlerts = (state: RootState): InsightResponse[] => state.insightSharing.insights?.filter((i) => i.type === 'alert') ?? [];
export const selectInsights = (state: RootState): InsightResponse[] => state.insightSharing.insights ?? [];

export default insightSharingSlice.reducer;
