import { AllLayoutAlgorithms, Layouts } from '@graphpolaris/shared/lib/graph-layout';
import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import { SchemaFromBackend, SchemaGraph, SchemaGraphInference, SchemaGraphology, SchemaGraphStats } from '../../schema';
import { SchemaViewState } from '../../schema/panel/Schema';
import { SchemaUtils } from '../../schema/schema-utils';
import type { RootState } from './store';

/**************************************************************** */

export const schemaConnectionTypeArray: Array<SchemaConnectionTypes> = ['connection', 'bezier', 'straight', 'step'];
export type SchemaConnectionTypes = 'connection' | 'bezier' | 'straight' | 'step';
export type SchemaSettings = {
  connectionType: SchemaConnectionTypes;
  layoutName: AllLayoutAlgorithms;
  animatedEdges: boolean;
  showMinimap: boolean;
  schemaViewState: SchemaViewState;
};

export type SchemaSliceI = {
  graph: SchemaGraph;
  graphInference: SchemaGraphInference;
  graphStats: SchemaGraphStats;
  loading: boolean;
  settings: SchemaSettings;
};

// Define the initial state using that type
export const initialState: SchemaSliceI = {
  graph: new SchemaGraphology().export(),
  graphInference: {
    nodes: {},
    edges: {},
  },
  graphStats: {
    nodes: {
      count: 0,
      stats: {},
    },
    edges: {
      count: 0,
      stats: {},
    },
  },
  loading: true,
  // layoutName: 'Cytoscape_fcose',
  settings: {
    connectionType: 'connection',
    // layoutName: Layouts.DAGRE,
    layoutName: Layouts.LISTINTERSECTED,
    animatedEdges: false,
    showMinimap: true,
    // schemaViewState: SchemaViewState.SchemaGraphS,
    schemaViewState: SchemaViewState.SchemaListS,
  },
};
export const schemaSlice = createSlice({
  name: 'schema',
  initialState,
  reducers: {
    setSchema: (state, action: PayloadAction<SchemaGraph>) => {
      if (action.payload === undefined) throw new Error('Schema is undefined');
      state.graph = action.payload;
      state.loading = false;
    },
    clearSchema: (state) => {
      state.graph = new SchemaGraphology().export();
    },

    readInSchemaFromBackend: (state, action: PayloadAction<SchemaFromBackend>) => {
      state.graph = SchemaUtils.schemaBackend2Graphology(action.payload).export();
    },

    setSchemaSettings: (state, action: PayloadAction<SchemaSettings>) => {
      state.settings = action.payload;
    },

    setSchemaAttributeDimensions: (state, action: PayloadAction<SchemaGraphInference>) => {
      state.graphInference = action.payload;
    },

    setSchemaAttributeInformation: (state, action: PayloadAction<SchemaGraphStats>) => {
      state.graphStats = action.payload;
    },

    setSchemaLoading: (state, action: PayloadAction<boolean>) => {
      state.loading = action.payload;
      if (!action.payload) state.graph = new SchemaGraphology().export();
    },
  },
});
export const {
  readInSchemaFromBackend,
  setSchema,
  setSchemaSettings,
  clearSchema,
  setSchemaAttributeDimensions,
  setSchemaAttributeInformation,
  setSchemaLoading,
} = schemaSlice.actions;

export const schema = (state: RootState) => state.schema;
export const schemaSettingsState = (state: RootState) => state.schema.settings;
export const schemaInferenceState = (state: RootState) => state.schema.graphInference;
export const schemaStatsState = (state: RootState) => state.schema.graphStats;

/**
 * Select the schema and convert it to a graphology object
 * */
export const toSchemaGraphology = (graph: SchemaGraph): SchemaGraphology => {
  // This is really weird but for some reason all the attributes appeared as read-only otherwise
  const ret = new SchemaGraphology();
  ret.import(SchemaGraphology.from(graph).export());
  return ret;
};

/**
 * Select the schema
 * */
export const schemaGraph = (state: RootState): SchemaGraph => {
  return state.schema.graph;
};

// /**
//  * selects the SchemaLayout enum
//  * @param {GraphLayout} state
//  * @returns {GraphLayout} enum of type GraphLayout
//  */
export const selectSchemaLayout = (state: RootState) => state.schema.settings.layoutName;

export default schemaSlice.reducer;
