import React, { useState, useEffect, useCallback } from 'react';
import { Button, DropdownContainer, DropdownItem, DropdownItemContainer, DropdownTrigger } from '../../components';
import { Tooltip, TooltipContent, TooltipProvider, TooltipTrigger } from '../../components/tooltip';
import { ControlContainer } from '../../components/controls';
import { Tabs, Tab } from '../../components/tabs';
import { useActiveSaveState, useActiveSaveStateAuthorization, useAppDispatch, useSessionCache, useVisualization } from '../../data-access';
import { addVisualization, removeVisualization, reorderVisState, setActiveVisualization } from '../../data-access/store/visualizationSlice';
import { VisualizationsConfig } from './config';
import { Visualizations } from './VisualizationPanel';

export default function VisualizationTabBar(props: { fullSize: () => void; exportImage: () => void; handleSelect: () => void }) {
  const { activeVisualizationIndex, openVisualizationArray } = useVisualization();
  const saveStateAuthorization = useActiveSaveStateAuthorization();
  const [open, setOpen] = useState(false);
  const dispatch = useAppDispatch();

  const handleDragStart = (e: React.DragEvent<HTMLDivElement>, i: number) => {
    e.dataTransfer.setData('text/plain', i.toString());
  };

  const handleDragOver = (e: React.DragEvent<HTMLButtonElement>) => {
    e.preventDefault();
  };

  const handleDrop = (e: React.DragEvent<HTMLButtonElement>, i: number) => {
    e.preventDefault();
    const draggedVisIndex = e.dataTransfer.getData('text/plain');
    dispatch(reorderVisState({ id: Number(draggedVisIndex), newPosition: i }));
  };

  const onSelect = async (id?: number) => {
    if (id === undefined) return;
    dispatch(setActiveVisualization(id));
  };

  const onDelete = (id: number) => {
    dispatch(removeVisualization(id));
    props.handleSelect();
  };

  useEffect(() => {
    const handleKeyDown = (e: KeyboardEvent) => {
      if ((e.ctrlKey || e.metaKey) && e.key === 's') {
        e.preventDefault();
        props.exportImage();
      }
    };

    window.addEventListener('keydown', handleKeyDown);

    return () => {
      window.removeEventListener('keydown', handleKeyDown);
    };
  }, [props]);

  return (
    <div
      className="absolute shrink-0 top-0 left-0 right-0 flex items-stretch justify-between h-7 bg-secondary-100 border-b border-secondary-200 max-w-full">
      <div className="flex items-center pl-2 pr-1 gap-1">
        <h1 className="text-xs font-semibold text-secondary-600 truncate">Visualization</h1>
        <TooltipProvider delayDuration={0}>
          <Tooltip>
            <TooltipTrigger>
              <DropdownContainer open={open} onOpenChange={setOpen}>
                <DropdownTrigger disabled={!saveStateAuthorization.database?.W} onClick={() => setOpen((v) => !v)}>
                  <Button as="a" variantType="secondary" variant="ghost" size="xs" iconComponent="icon-[ic--baseline-add]" />
                </DropdownTrigger>
                <DropdownItemContainer>
                  {Object.values(VisualizationsConfig).map(({ id, displayName, icons }) => (
                    <DropdownItem
                      key={id}
                      value={id}
                      label={displayName}
                      className="flex items-center gap-2"
                      onClick={async () => {
                        const component = await Visualizations[id]();
                        dispatch(addVisualization({ ...component.default.settings, name: displayName, id }));
                        setOpen(false);
                      }}
                    >
                      {icons.sm && <icons.sm className="h-4 w-4" />}
                    </DropdownItem>
                  ))}
                </DropdownItemContainer>
              </DropdownContainer>
            </TooltipTrigger>
            <TooltipContent>
              <p>Add visualization</p>
            </TooltipContent>
          </Tooltip>
        </TooltipProvider>
      </div>

      <Tabs>
        {openVisualizationArray.map((vis, i) => {
          const isActive = activeVisualizationIndex === i;
          const config = VisualizationsConfig[vis.id];
          const IconComponent = config?.icons.sm;

          return (
            <Tab
              key={i}
              activeTab={isActive}
              text={vis.name}
              IconComponent={IconComponent}
              className="group"
              onClick={() => onSelect(i)}
              onDragStart={(e) => handleDragStart(e, i)}
              onDragOver={(e) => handleDragOver(e)}
              onDrop={(e) => handleDrop(e, i)}
              draggable
            >
              <Button
                variantType="secondary"
                variant="ghost"
                disabled={!saveStateAuthorization.database?.W}
                rounded
                size="2xs"
                iconComponent="icon-[ic--baseline-close]"
                className={!isActive ? 'opacity-0 group-hover:opacity-100' : ''}
                onClick={(e) => {
                  e.stopPropagation();
                  onDelete(i);
                }}
              />
            </Tab>
          );
        })}
      </Tabs>
      <div className="shrink-0 sticky right-0 px-0.5 ml-auto flex">
        <ControlContainer>
          {openVisualizationArray.length > 0 && (
            <TooltipProvider delayDuration={0}>
              <Tooltip>
                <TooltipTrigger asChild>
                  <Button
                    variantType="secondary"
                    variant="ghost"
                    size="xs"
                    iconComponent="icon-[ic--baseline-fullscreen]"
                    onClick={props.fullSize}
                  />
                </TooltipTrigger>
                <TooltipContent>
                  <p>Full screen</p>
                </TooltipContent>
              </Tooltip>
              <Tooltip>
                <TooltipTrigger asChild>
                  <Button
                    variantType="secondary"
                    variant="ghost"
                    size="xs"
                    iconComponent="icon-[ic--baseline-camera-alt]"
                    onClick={props.exportImage}
                  />
                </TooltipTrigger>
                <TooltipContent>
                  <p>Export image [Ctrl+S]</p>
                </TooltipContent>
              </Tooltip>
            </TooltipProvider>
          )}
        </ControlContainer>
      </div>
    </div>
  );
}
